package ax;

import waba.ui.Control;
import waba.ui.MainWindow;
import waba.ui.Event;
import waba.ui.ControlEvent;
import waba.ui.PenEvent;
import waba.fx.Graphics;
import waba.fx.Color;
import waba.fx.FontMetrics;
import waba.fx.Font;
import waba.fx.Image;

/**
 * {^Rg[
 */
public class JButton extends Control {

    protected String text;
    protected Font font;
    protected boolean armed;

    public JButton(String text) {
        this.text = text;
        this.font = AxGlobals.FONT_BOLD;
    }

    public void setFont(Font font) {
        this.font = font;
    }

    public Font getFont() {
        return font;
    }

    public void setText(String text) {
        this.text = text;
        repaint();
    }

    public String getText() {
        return text;
    }

    public void onEvent(Event event) {
        if (event.type == PenEvent.PEN_DOWN) {
            armed = true;
            repaint();
        } else if (event.type == PenEvent.PEN_UP) {
            armed = false;
            repaint();
            PenEvent pe = (PenEvent)event;
            if (pe.x >= 0 && pe.x < this.width &&
                             pe.y >= 0 && pe.y < this.height) {
                postEvent(new ControlEvent(ControlEvent.PRESSED, this));
            } else if (event.type == PenEvent.PEN_DRAG) {
                boolean lArmed = false;
                if (pe.x >= 0 && pe.x < this.width &&
                    pe.y >= 0 && pe.y < this.height) {
                    lArmed = true;
                }
                if (armed != lArmed) {
                    armed = lArmed;
                    repaint();
                }
            }
        }
    }

    protected static void drawJButton(Graphics g, boolean armed, 
                                      int width,  int height) {
        if (armed) {
            setColor(g, AxGlobals.COLOR_PUSHED);
            g.fillRect(0, 0, width - 1, height - 1);
        } else {
            setColor(g, AxGlobals.COLOR_BACK);
            g.fillRect(0, 0, width, height);
        }

        setColor(g, AxGlobals.COLOR_SHADOW);
        g.drawRect(0, 0, width - 1, height - 1);

        setColor(g, AxGlobals.COLOR_WHITE);
        g.drawRect(1, 1, width - 1, height - 1);

        if (armed) {
            setColor(g, AxGlobals.COLOR_PUSHED);
            g.drawLine(1, 1, 1, height - 2);
            g.drawLine(1, 1, width - 2, 1);
        } else {
            setColor(g, AxGlobals.COLOR_BACK);
            g.fillRect(width - 2, 1, 1, 1);
            g.fillRect(1, height - 2, 1, 1);
        }
    }

    protected static void setColor(Graphics g, Color color) {
        g.setColor(color.getRed(), color.getGreen(), color.getBlue());
    }

    public void onPaint(Graphics g) {

        Image image = new Image(width, height);
        Graphics offScreen = new Graphics(image);

        drawJButton(offScreen, armed, this.width, this.height);

        setColor(offScreen, AxGlobals.COLOR_BLACK);

        offScreen.setFont(font);
        FontMetrics fm = getFontMetrics(font);

        int x = (this.width - fm.getTextWidth(text)) / 2;
        int y = (this.height - fm.getHeight()) / 2;

        offScreen.drawText(text, x, y);
        g.drawImage(image, 0, 0);
    }
}
