package ax;

import waba.fx.Font;
import waba.fx.FontMetrics;
import waba.fx.Graphics;
import waba.fx.Color;
import waba.fx.Image;
import waba.ui.Control;
import waba.ui.Event;
import waba.ui.ControlEvent;
import waba.ui.PenEvent;
import waba.util.Vector;

import ax.event.Selectable;
import ax.event.SelectionListener;

/**
 * `FbN{bNXRg[
 */
public class JCheckBox extends Control implements Selectable {

    protected String text;
    protected Font font;

    protected boolean armed    = false;
    protected boolean selected = false;

    protected Vector listeners = new Vector();

    protected static int boxSize = 12;

    public JCheckBox(String text) {
        this.text = text;
        this.font = AxGlobals.FONT_BOLD;
    }

    public void setFont(Font font) {
        this.font = font;
    }

    public Font getFont() {
        return font;
    }

    public void setText(String text) {
        this.text   = text;
        repaint();
    }

    public String getText() {
        return text;
    }

    public void setSelected(boolean selected) {
        this.selected = selected;
        repaint();
    }

    public boolean getSelected() {
        return selected;
    }

    public void onEvent(Event event) {
        if (event.type == PenEvent.PEN_DOWN) {
            armed = true;
            selected = !selected;
            if (selected) {
                fireOnSelect();
            }
            repaint();
        } else if (event.type == PenEvent.PEN_UP) {
            armed = false;
            repaint();
            PenEvent pe = (PenEvent)event;
            if (pe.x >= 0 && pe.x < this.width &&
                             pe.y >= 0 && pe.y < this.height) {
                postEvent(new ControlEvent(ControlEvent.PRESSED, this));
            } else if (event.type == PenEvent.PEN_DRAG) {
                boolean lArmed = false;
                if (pe.x >= 0 && pe.x < this.width &&
                    pe.y >= 0 && pe.y < this.height) {
                    lArmed = true;
                }
                if (armed != lArmed) {
                    armed = lArmed;
                    repaint();
                }
            }
        }
    }

    protected static void drawCheckBox(Graphics g, boolean armed, 
                             boolean selected, int width,  int height) {

        setColor(g, AxGlobals.COLOR_BACK);
        g.fillRect(0, 0, boxSize+1, boxSize+1);

        setColor(g, AxGlobals.COLOR_SHADOW);
        g.drawRect(0, 0, boxSize, boxSize);

        setColor(g, AxGlobals.COLOR_WHITE);
        g.drawRect(1, 1, boxSize, boxSize);

        if (selected) {
            setColor(g, AxGlobals.COLOR_BLACK);
            g.fillRect(3, 5, 2, 5);
            g.drawLine(5, 7, 5, 8);
            g.drawLine(6, 6, 6, 7);
            g.drawLine(7, 5, 7, 6);
            g.drawLine(8, 4, 8, 5);
            g.drawLine(9, 3, 9, 4);
        }

        setColor(g, AxGlobals.COLOR_BACK);
        g.fillRect(boxSize - 1, 1, 1, 1);
        g.fillRect(1, boxSize - 1, 1, 1);

    }

    protected static void setColor(Graphics g, Color color) {
        g.setColor(color.getRed(), color.getGreen(), color.getBlue());
    }

    public void onPaint(Graphics g) {

        Image image = new Image(boxSize+1, boxSize+1);
        Graphics offScreen = new Graphics(image);

        drawCheckBox(offScreen, armed, selected, this.width, this.height);
        g.drawImage(image, 0, 0);

        setColor(g, AxGlobals.COLOR_BLACK);

        g.setFont(font);
        FontMetrics fm = getFontMetrics(font);

        int x = 17;
        int y = (this.height - fm.getHeight()) / 2 - 3;

        g.drawText(text, x, y);

    }

    public void addSelectionListener(SelectionListener listener) {
        listeners.add(listener);
    }

    protected void fireOnSelect() {
        ControlEvent event = new ControlEvent(ControlEvent.PRESSED, this);
        for (int i = 0; i < listeners.getCount(); i++) {
            ((SelectionListener)listeners.get(i)).onSelect(event);
        }
    }
}
