package ax;

import waba.fx.Font;
import waba.fx.FontMetrics;
import waba.fx.Graphics;
import waba.fx.Color;
import waba.fx.Image;
import waba.ui.Control;
import waba.ui.Event;
import waba.ui.ControlEvent;
import waba.ui.PenEvent;
import waba.util.Vector;
import ax.event.Selectable;
import ax.event.SelectionListener;

/**
 * WI{^Rg[
 */
public class JRadioButton extends Control implements Selectable {

    protected String text;
    protected Font font;

    protected boolean armed    = false;
    protected boolean selected = false;

    protected Vector listeners = new Vector();

    public JRadioButton(String text) {
        this.text = text;
        this.font = AxGlobals.FONT_BOLD;
    }

    public void setFont(Font font) {
        this.font = font;
    }

    public Font getFont() {
        return font;
    }

    public void setText(String text) {
        this.text = text;
        repaint();
    }

    public String getText() {
        return text;
    }

    public void setSelected(boolean selected) {
        this.selected = selected;
        repaint();
    }

    public boolean getSelected() {
        return selected;
    }

    public void onEvent(Event event) {
        if (event.type == PenEvent.PEN_DOWN) {
            armed = true;
            selected = true;
            fireOnSelect();
            repaint();
        } else if (event.type == PenEvent.PEN_UP) {
            armed = false;
            repaint();
            PenEvent pe = (PenEvent)event;
            if (pe.x >= 0 && pe.x < this.width &&
                             pe.y >= 0 && pe.y < this.height) {
                postEvent(new ControlEvent(ControlEvent.PRESSED, this));
            } else if (event.type == PenEvent.PEN_DRAG) {
                boolean lArmed = false;
                if (pe.x >= 0 && pe.x < this.width &&
                    pe.y >= 0 && pe.y < this.height) {
                    lArmed = true;
                }
                if (armed != lArmed) {
                    armed = lArmed;
                    repaint();
                }
            }
        }
    }

    protected static void drawRadioButton(Graphics g, boolean armed, 
                             boolean selected, int width,  int height) {
        setColor(g, AxGlobals.COLOR_BACK);
        g.fillRect(0, 0, width, height);

        setColor(g, AxGlobals.COLOR_SHADOW);
        g.drawLine(4, 0, 7, 0);
        g.drawLine(2, 1, 3, 1);
        g.drawLine(8, 1, 9, 1);

        g.drawLine( 1, 2,  1, 3);
        g.drawLine(10, 2, 10, 3);

        g.drawLine( 0, 4,  0, 7);
        g.drawLine(11, 4, 11, 7);

        g.drawLine( 1, 8,  1, 9);
        g.drawLine(10, 8, 10, 9);

        g.drawLine( 2, 10,  3, 10);
        g.drawLine( 8, 10,  9, 10);

        g.drawLine( 3, 11,  7, 11);

        setColor(g, AxGlobals.COLOR_WHITE);
        g.drawLine(4, 1, 7, 1);
        g.drawLine(2, 2, 3, 2);
        g.drawLine(2, 2, 2, 3);
        g.drawLine(8, 2, 9, 2);

        g.fillRect(10, 1, 1, 1);
        g.drawLine(11, 2, 11, 3);

        g.drawLine(1,  4,  1, 7);
        g.drawLine(12, 4, 12, 7);

        g.drawLine(2,  8,  2, 9);
        g.drawLine(11, 8, 11, 9);

        g.drawLine(2, 11, 3, 11);
        g.drawLine(8, 11, 9, 11);

        g.drawLine(4, 12, 7, 12);
        g.fillRect(10, 10, 1, 1);

        if (selected) {
            setColor(g, AxGlobals.COLOR_BLACK);
            g.fillRect( 4, 3,  4, 6);
            g.drawLine( 3, 4,  3, 7);
            g.drawLine( 8, 4,  8, 7);
        }
    }

    protected static void setColor(Graphics graphics, Color color) {
        graphics.setColor(color.getRed(), color.getGreen(), color.getBlue());
    }

    public void onPaint(Graphics g) {

        Image image = new Image(width, height);
        Graphics offScreen = new Graphics(image);

        drawRadioButton(offScreen, armed, selected, this.width, this.height);

        setColor(offScreen, AxGlobals.COLOR_BLACK);

        offScreen.setFont(font);
        FontMetrics fm = getFontMetrics(font);

        int x = 17;
        int y = (this.height - fm.getHeight()) / 2 - 3;

        offScreen.drawText(text, x, y);
        g.drawImage(image, 0, 0);
    }

    public void addSelectionListener(SelectionListener listener) {
        listeners.add(listener);
    }

    protected void fireOnSelect() {
        ControlEvent event = new ControlEvent(ControlEvent.PRESSED, this);
        for (int i = 0; i < listeners.getCount(); i++) {
            ((SelectionListener)listeners.get(i)).onSelect(event);
        }
    }

}
